@@ -6,6 +6,7 @@ use Mouse;
 use Lemonldap::NG::Manager::Build::Attributes;
 use Lemonldap::NG::Manager::Build::Tree;
 use Lemonldap::NG::Manager::Build::CTrees;
+use Lemonldap::NG::Manager::Build::PortalConstants;
 use Lemonldap::NG::Manager::Conf::Zero;
 use Data::Dumper;
 use Regexp::Assemble;
@@ -13,14 +14,16 @@ use JSON;
 use Getopt::Std;
 use IO::String;
 
-has structFile            => ( isa => 'Str', is => 'ro', required => 1 );
-has confTreeFile          => ( isa => 'Str', is => 'ro', required => 1 );
-has managerConstantsFile  => ( isa => 'Str', is => 'ro', required => 1 );
-has managerAttributesFile => ( isa => 'Str', is => 'ro', required => 1 );
-has defaultValuesFile     => ( isa => 'Str', is => 'ro', required => 1 );
-has confConstantsFile     => ( isa => 'Str', is => 'ro', required => 1 );
-has firstLmConfFile       => ( isa => 'Str', is => 'ro', required => 1 );
-has reverseTreeFile       => ( isa => 'Str', is => 'ro', required => 1 );
+has structFile                 => ( isa => 'Str', is => 'ro', required => 1 );
+has confTreeFile               => ( isa => 'Str', is => 'ro', required => 1 );
+has managerConstantsFile       => ( isa => 'Str', is => 'ro', required => 1 );
+has managerAttributesFile      => ( isa => 'Str', is => 'ro', required => 1 );
+has defaultValuesFile          => ( isa => 'Str', is => 'ro', required => 1 );
+has confConstantsFile          => ( isa => 'Str', is => 'ro', required => 1 );
+has firstLmConfFile            => ( isa => 'Str', is => 'ro', required => 1 );
+has reverseTreeFile            => ( isa => 'Str', is => 'ro', required => 1 );
+has portalConstantsFile        => ( isa => 'Str', is => 'ro', required => 1 );
+has handlerStatusConstantsFile => ( isa => 'Str', is => 'ro', required => 1 );
 
 my @managerAttrKeys = qw(keyTest keyMsgFail select type test msgFail default);
 my $format          = 'Creating %-69s: ';
@@ -181,11 +184,11 @@ our \@EXPORT      = ( \@{ \$EXPORT_TAGS{'all'} } );
 our \$specialNodeHash = {
     virtualHosts         => [qw(exportedHeaders locationRules post vhostOptions)],
     samlIDPMetaDataNodes => [qw(samlIDPMetaDataXML samlIDPMetaDataExportedAttributes samlIDPMetaDataOptions)],
-    samlSPMetaDataNodes  => [qw(samlSPMetaDataXML samlSPMetaDataExportedAttributes samlSPMetaDataOptions)],
+    samlSPMetaDataNodes  => [qw(samlSPMetaDataXML samlSPMetaDataExportedAttributes samlSPMetaDataOptions samlSPMetaDataMacros)],
     oidcOPMetaDataNodes  => [qw(oidcOPMetaDataJSON oidcOPMetaDataJWKS oidcOPMetaDataOptions oidcOPMetaDataExportedVars)],
-    oidcRPMetaDataNodes  => [qw(oidcRPMetaDataOptions oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims)],
+    oidcRPMetaDataNodes  => [qw(oidcRPMetaDataOptions oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims oidcRPMetaDataMacros)],
     casSrvMetaDataNodes  => [qw(casSrvMetaDataOptions casSrvMetaDataExportedVars)],
-    casAppMetaDataNodes  => [qw(casAppMetaDataOptions casAppMetaDataExportedVars)],
+    casAppMetaDataNodes  => [qw(casAppMetaDataOptions casAppMetaDataExportedVars casAppMetaDataMacros)],
 };
 
 EOF
@@ -274,11 +277,13 @@ $defaultAttr}
         exportedHeaders locationRules post vhostOptions
         samlIDPMetaDataXML samlIDPMetaDataExportedAttributes
         samlIDPMetaDataOptions samlSPMetaDataXML
-        samlSPMetaDataExportedAttributes samlSPMetaDataOptions
-        oidcOPMetaDataJSON oidcOPMetaDataJWKS oidcOPMetaDataOptions
+        samlSPMetaDataExportedAttributes samlSPMetaDataMacros
+        samlSPMetaDataOptions oidcOPMetaDataJSON
+        oidcOPMetaDataJWKS oidcOPMetaDataOptions
         oidcOPMetaDataExportedVars oidcRPMetaDataOptions
         oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims
-        casAppMetaDataExportedVars casAppMetaDataOptions
+        oidcRPMetaDataMacros casAppMetaDataExportedVars
+        casAppMetaDataOptions casAppMetaDataMacros
         casSrvMetaDataExportedVars casSrvMetaDataOptions
         )
       )
@@ -370,6 +375,10 @@ EOF
         } keys(%$attributes)
     };
     $managerAttr = mydump( $managerAttr, 'attributes' );
+    my $managerSub =
+      Dumper( \&Lemonldap::NG::Manager::Build::Attributes::perlExpr );
+    $managerSub =~ s/\$VAR1 = sub/sub perlExpr/s;
+    $managerSub =~ s/^\s*(?:use strict;|package .*?;|)\n//gm;
     my $managerTypes =
       mydump( Lemonldap::NG::Manager::Build::Attributes::types(), 'types' );
     $managerAttr = "# This file is generated by $module. Don't modify it by hand
@@ -377,6 +386,8 @@ package Lemonldap::NG::Manager::Attributes;
 
 our \$VERSION = '$Lemonldap::NG::Manager::Build::Attributes::VERSION';
 
+$managerSub
+
 $managerTypes}
 
 $managerAttr}
@@ -402,7 +413,9 @@ $managerAttr}
     $jsonEnc->pretty(0);
     print F $jsonEnc->encode($reverseScanResult);
     close F;
+
     print STDERR "done\n";
+    $self->buildPortalConstants();
 }
 
 sub buildZeroConf {
@@ -420,6 +433,106 @@ sub buildZeroConf {
     print STDERR "done\n";
 }
 
+sub buildPortalConstants() {
+    my $self = shift;
+
+    my %portalConstants =
+      %{ Lemonldap::NG::Manager::Build::PortalConstants::portalConstants() };
+    my %reverseConstants = reverse %portalConstants;
+    die "Duplicate value in portal constants"
+      unless %reverseConstants == %portalConstants;
+
+    printf STDERR $format, $self->portalConstantsFile;
+    open( F, '>', $self->portalConstantsFile ) or die($!);
+    my $content = <<EOF;
+# This file is generated by $module. Don't modify it by hand
+package Lemonldap::NG::Portal::Main::Constants;
+
+use strict;
+use Exporter 'import';
+
+our \$VERSION = '$Lemonldap::NG::Manager::Build::Attributes::VERSION';
+
+use constant HANDLER => 'Lemonldap::NG::Handler::PSGI::Main';
+use constant {
+EOF
+    for my $pe (
+        sort { $portalConstants{$a} <=> $portalConstants{$b} }
+        keys %portalConstants
+      )
+    {
+        my $str = $portalConstants{$pe};
+        $content .= "    $pe => $str,\n";
+    }
+
+    my $exports = join ", ", map { "'$_'" }
+      sort { $portalConstants{$a} <=> $portalConstants{$b} }
+      keys %portalConstants;
+
+    my $portalConstsStr .= mydump( \%reverseConstants, 'portalConsts' );
+    $content .= <<EOF;
+};
+
+$portalConstsStr
+}
+
+# EXPORTER PARAMETERS
+our \@EXPORT_OK = ( 'portalConsts', 'HANDLER', $exports );
+our %EXPORT_TAGS = ( 'all' => [ \@EXPORT_OK, 'import' ], );
+
+our \@EXPORT = qw(import PE_OK);
+
+1;
+EOF
+
+    my $dst;
+    eval {
+        Perl::Tidy::perltidy(
+            source      => IO::String->new($content),
+            destination => \$dst
+        );
+    };
+    $dst = $content if ($@);
+    open( F, '>', $self->portalConstantsFile ) or die($!);
+    print F $dst;
+    close F;
+    print STDERR "done\n";
+
+    printf STDERR $format, $self->handlerStatusConstantsFile;
+
+    # Handler Status file
+    $content = <<EOF;
+# This file is generated by $module. Don't modify it by hand
+package Lemonldap::NG::Handler::Lib::StatusConstants;
+
+use strict;
+use Exporter 'import';
+
+our \$VERSION = '$Lemonldap::NG::Manager::Build::Attributes::VERSION';
+
+$portalConstsStr
+}
+
+# EXPORTER PARAMETERS
+our \@EXPORT_OK = ( 'portalConsts' );
+our %EXPORT_TAGS = ( 'all' => [ \@EXPORT_OK, 'import' ], );
+
+1;
+EOF
+
+    eval {
+        Perl::Tidy::perltidy(
+            source      => IO::String->new($content),
+            destination => \$dst
+        );
+    };
+    $dst = $content if ($@);
+    open( F, '>', $self->handlerStatusConstantsFile ) or die($!);
+    print F $dst;
+    close F;
+    print STDERR "done\n";
+}
+
 sub mydump {
     my ( $obj, $subname ) = @_;
     my $t = Dumper($obj);
@@ -435,7 +548,7 @@ sub scanTree {
         die 'Not an array';
     }
     $prefix //= '';
-    my $ord = -1;
+    my $ord      = -1;
     my $nodeName = $path ? '_nodes' : 'data';
     foreach my $leaf (@$tree) {
         $ord++;
@@ -457,7 +570,7 @@ sub scanTree {
         # Subnode
         elsif ( ref($leaf) ) {
             $jleaf->{title} = $jleaf->{id} = $leaf->{title};
-            $jleaf->{type} = $leaf->{form} if ( $leaf->{form} );
+            $jleaf->{type}  = $leaf->{form} if ( $leaf->{form} );
             if ( $leaf->{title} =~ /^((?:oidc|saml|cas)Service)MetaData$/ ) {
                 no strict 'refs';
                 my @tmp = $self->scanLeaf( $leaf->{nodes} );
@@ -492,7 +605,7 @@ sub scanTree {
                     }
                 }
             }
-            $jleaf->{help} = $leaf->{help} if ( $leaf->{help} );
+            $jleaf->{help}          = $leaf->{help} if ( $leaf->{help} );
             $jleaf->{_nodes_filter} = $leaf->{nodes_filter}
               if ( $leaf->{nodes_filter} );
             push @$json, $jleaf;
@@ -738,7 +851,7 @@ must contain an array of { k => <keyName>, v => <display name> } hashref
 
 =item documentation (recommended):
 
-some words for other developpers
+some words for other developers
 
 =item test (optional):
 
