@@ -14,7 +14,7 @@
 
 package Lemonldap::NG::Manager::Build::CTrees;
 
-our $VERSION = '2.0.4';
+our $VERSION = '2.0.8';
 
 sub cTrees {
     return {
@@ -30,6 +30,7 @@ sub cTrees {
                     'vhostPort',        'vhostHttps',
                     'vhostMaintenance', 'vhostAliases',
                     'vhostType',        'vhostAuthnLevel',
+                    'vhostServiceTokenTTL'
                 ],
             },
         ],
@@ -137,7 +138,8 @@ sub cTrees {
                         ]
                     }
                 ]
-            }
+            },
+            "samlSPMetaDataMacros",
         ],
         oidcOPMetaDataNode => [
             'oidcOPMetaDataJSON',
@@ -189,22 +191,49 @@ sub cTrees {
             'oidcRPMetaDataOptionsExtraClaims',
             {
                 title => 'oidcRPMetaDataOptions',
+                help  => 'idpopenidconnect.html#options',
                 nodes => [ {
-                        title => 'oidcRPMetaDataOptionsAuthentication',
+                        title => 'oidcRPMetaDataOptionsBasic',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcRPMetaDataOptionsClientID',
                             'oidcRPMetaDataOptionsClientSecret',
                             'oidcRPMetaDataOptionsPublic',
+                            'oidcRPMetaDataOptionsRedirectUris',
+                        ]
+                    },
+                    {
+                        title => 'oidcRPMetaDataOptionsAdvanced',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsBypassConsent',
+                            'oidcRPMetaDataOptionsUserIDAttr',
+                            'oidcRPMetaDataOptionsIDTokenForceClaims',
+                            'oidcRPMetaDataOptionsAdditionalAudiences',
+                            'oidcRPMetaDataOptionsRefreshToken',
+                        ]
+                    },
+                    {
+                        title => 'security',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsIDTokenSignAlg',
                             'oidcRPMetaDataOptionsRequirePKCE',
+                            'oidcRPMetaDataOptionsAllowOffline',
+                            'oidcRPMetaDataOptionsAllowPasswordGrant',
+                            'oidcRPMetaDataOptionsRule',
+                        ]
+                    },
+                    {
+                        title => 'oidcRPMetaDataOptionsTimeouts',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsAuthorizationCodeExpiration',
+                            'oidcRPMetaDataOptionsIDTokenExpiration',
+                            'oidcRPMetaDataOptionsAccessTokenExpiration',
+                            'oidcRPMetaDataOptionsOfflineSessionExpiration',
                         ]
                     },
-                    'oidcRPMetaDataOptionsUserIDAttr',
-                    'oidcRPMetaDataOptionsIDTokenSignAlg',
-                    'oidcRPMetaDataOptionsIDTokenExpiration',
-                    'oidcRPMetaDataOptionsAccessTokenExpiration',
-                    'oidcRPMetaDataOptionsRedirectUris',
-                    'oidcRPMetaDataOptionsBypassConsent',
                     {
                         title => 'logout',
                         form  => 'simpleInputContainer',
@@ -215,9 +244,9 @@ sub cTrees {
                             'oidcRPMetaDataOptionsLogoutSessionRequired',
                         ]
                     },
-                    'oidcRPMetaDataOptionsRule',
                 ]
             },
+            'oidcRPMetaDataMacros',
             {
                 title => 'oidcRPMetaDataOptionsDisplay',
                 form  => 'simpleInputContainer',
@@ -249,7 +278,9 @@ sub cTrees {
                 ]
             },
         ],
-        casAppMetaDataNode => [ {
+        casAppMetaDataNode => [
+            'casAppMetaDataExportedVars',
+            {
                 title => 'casAppMetaDataOptions',
                 form  => 'simpleInputContainer',
                 nodes => [
@@ -258,7 +289,7 @@ sub cTrees {
                     'casAppMetaDataOptionsRule'
                 ]
             },
-            'casAppMetaDataExportedVars',
+            'casAppMetaDataMacros',
         ],
     };
 }
