@@ -26,11 +26,11 @@ use JSON 'to_json';
 use Lemonldap::NG::Common::Conf::ReConstants;
 use Lemonldap::NG::Manager::Attributes;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Common::Conf::Compact';
 
-# High debugging for developpers, set this to 1
+# High debugging for developers, set this to 1
 use constant HIGHDEBUG => 0;
 
 # Messages storage
@@ -51,7 +51,10 @@ has changes => ( is => 'rw', isa => 'ArrayRef', default => sub { return [] } );
 has message => (
     is      => 'rw',
     isa     => 'Str',
-    default => '',
+    lazy    => 1,
+    default => sub {
+        return join( ', ', map { $_->{message} } @{ $_[0]->errors } );
+    },
     trigger => sub {
         hdebug( "Message becomes " . $_[0]->{message} );
     }
@@ -94,18 +97,19 @@ sub hdebug {
 # Main method
 #@return result
 sub check {
-    my $self = shift;
+    my ( $self, $localConf ) = @_;
+
     hdebug("# check()");
     unless ( $self->newConf ) {
         return 0 unless ( $self->scanTree );
     }
-    unless ( $self->testNewConf ) {
+    unless ( $self->testNewConf($localConf) ) {
         hdebug("  testNewConf() failed");
         return 0;
     }
     my $separator = $self->newConf->{multiValuesSeparator} || '; ';
     hdebug("  tests succeed");
-    my %conf          = %{ $self->newConf() };
+    my %conf          = %{ $self->newConf };
     my %compactedConf = %{ $self->compactConf( $self->newConf ) };
     my @removedKeys   = ();
     unless ( $self->confChanged ) {
@@ -113,20 +117,20 @@ sub check {
         $self->message('__confNotChanged__');
         return 0;
     }
-    unless ( $self->newConf->{dontCompactConf} ) {
-        foreach ( sort keys %conf ) {
-            push @removedKeys, $_ unless exists $compactedConf{$_};
-        }
-    }
+
+    # Return removed keys if conf compacted
+    @removedKeys = map { exists $compactedConf{$_} ? () : $_ } sort keys %conf
+      if ( $self->newConf->{compactConf} );
     push @{ $self->changes },
       (
-        $self->{newConf}->{dontCompactConf}
-        ? { confCompacted => '0' }
-        : {
+        $self->{newConf}->{compactConf}
+        ? {
             confCompacted => '1',
             removedKeys   => join( $separator, @removedKeys )
-        }
+          }
+        : { confCompacted => '0' }
       );
+
     return 1;
 }
 
@@ -264,6 +268,8 @@ sub _scanNodes {
                               $leaf->{comment}
                               ? "(?#$leaf->{comment})$leaf->{re}"
                               : $leaf->{re};
+                            $k .= "(?#AuthnLevel=$leaf->{level})"
+                              if $leaf->{level};
                             $self->set( $target, $key, $k, $leaf->{data} );
                         }
                         else {
@@ -331,7 +337,9 @@ sub _scanNodes {
                     hdebug("  SAML data is an array, serializing");
                     $leaf->{data} = join ';', @{ $leaf->{data} };
                 }
-                if ( $target =~ /^saml(?:S|ID)PMetaDataExportedAttributes$/ ) {
+                if ( $target =~
+                    /^saml(?:S|ID)PMetaData(?:ExportedAttributes|Macros)$/ )
+                {
                     if ( $leaf->{cnodes} ) {
                         hdebug("  $target: unopened node");
                         $self->newConf->{$target}->{$key} =
@@ -391,7 +399,9 @@ sub _scanNodes {
                     hdebug("  $target");
                     $self->set( $target, $key, $leaf->{data} );
                 }
-                elsif ( $target =~ /^oidc(?:O|R)PMetaDataExportedVars$/ ) {
+                elsif (
+                    $target =~ /^oidc(?:O|R)PMetaData(?:ExportedVars|Macros)$/ )
+                {
                     hdebug("  $target");
                     if ( $leaf->{cnodes} ) {
                         hdebug('    unopened');
@@ -460,7 +470,9 @@ sub _scanNodes {
                     $self->_scanNodes($subNodes);
                     $self->set( $target, $key, $leaf->{title}, $leaf->{data} );
                 }
-                elsif ( $target =~ /^cas(?:App|Srv)MetaDataExportedVars$/ ) {
+                elsif ( $target =~
+                    /^cas(?:App|Srv)MetaData(?:ExportedVars|Macros)$/ )
+                {
                     hdebug("  $target");
                     if ( $leaf->{cnodes} ) {
                         hdebug('    unopened');
@@ -668,7 +680,7 @@ sub _scanNodes {
             }
 
             # Create new apps
-            else {
+            if ( $leaf->{type} eq 'menuApp' ) {
                 hdebug('  new app');
                 $knownCat->{__id}++;
                 $cn->{$newapp} = {
@@ -819,7 +831,7 @@ sub _scanNodes {
                     foreach my $node ( @{ $leaf->{nodes} } ) {
                         my $tmp;
                         $tmp->{$_} = $node->{data}->{$_}
-                          foreach (qw(type rule logo label));
+                          foreach (qw(type rule logo level label));
                         $tmp->{over} = {};
                         foreach ( @{ $node->{data}->{over} } ) {
                             $tmp->{over}->{ $_->[0] } = $_->[1];
@@ -846,9 +858,14 @@ sub _scanNodes {
 
                         # authChoiceModules
                         if ( $name eq 'authChoiceModules' ) {
-                            hdebug('     combModules');
-                            $n->{data}->[5] ||= {};
-                            $n->{data}->[5] = to_json( $n->{data}->[5] );
+                            hdebug('     authChoiceModules');
+                            if ( ref( $n->{data}->[5] ) eq 'ARRAY' ) {
+                                $n->{data}->[5] = to_json(
+                                    { map { @$_ } @{ $n->{data}->[5] } } );
+                            }
+                            else {
+                                $n->{data}->[5] = '{}';
+                            }
                         }
 
                         $n->{data} = join ';', @{ $n->{data} };
@@ -1078,9 +1095,11 @@ sub defaultValue {
 #
 #@return true if tests succeed
 sub testNewConf {
-    my $self = shift;
+    my ( $self, $localConf ) = @_;
+
     hdebug('# testNewConf()');
-    return $self->_unitTest( $self->newConf(), '' ) && $self->_globalTest();
+    return $self->_unitTest( $self->newConf(), $localConf )
+      && $self->_globalTest($localConf);
 }
 
 ##@method private boolean _unitTest()
@@ -1088,31 +1107,49 @@ sub testNewConf {
 #
 #@return true if tests succeed
 sub _unitTest {
-    my ( $self, $conf ) = @_;
+    my ( $self, $conf, $localConf ) = @_;
     hdebug('# _unitTest()');
     my $types = &Lemonldap::NG::Manager::Attributes::types();
     my $attrs = &Lemonldap::NG::Manager::Attributes::attributes();
     my $res   = 1;
+
     foreach my $key ( keys %$conf ) {
-        if (    $self->{skippedUnitTests}
-            and $self->{skippedUnitTests} =~ /\b$key\b/ )
+        if (    $localConf
+            and $localConf->{skippedUnitTests}
+            and $localConf->{skippedUnitTests} =~ /\b$key\b/ )
         {
-            $self->logger->debug("Ignore test for $key");
+            $localConf->logger->debug("-> Ignore test for $key\n");
             next;
         }
         hdebug("Testing $key");
         my $attr = $attrs->{$key};
-        my $type = $types->{ $attr->{type} };
+        my $type = $types->{ $attr->{type} } if $attr;
         unless ( $type or $attr->{test} ) {
-            print STDERR "Unknown attribute $key, deleting it\n";
+            $localConf->logger->debug("Unknown attribute $key, deleting it\n")
+              if $localConf;
             delete $conf->{$key};
             next;
         }
 
-        if ( $attr->{type} and $attr->{type} eq 'subContainer' ) {
+        # Vhost, CAS, SAML, OIDC options
+        if ( $key =~
+/^(?^:(?:(?:(?:saml(?:ID|S)|oidc[OR])P|cas(?:App|Srv))MetaData|vhost)Options)$/
+          )
+        {
 
-            # TODO Recursive for SAML/OIDC nodes
+            # Iterate on vhost names, or saml/cas/oidc configuration keys
+            for my $vhost ( keys %{ $conf->{$key} } ) {
+                my $options = $conf->{$key}->{$vhost};
+                if ( ref($options) eq "HASH" ) {
+
+                    # Recurse on option list,
+                    # FIXME this does check for oidcRPMetaDataOptionsXXX appearing under
+                    # samlSPMetadataOptions
+                    $res = 0 unless $self->_unitTest( $options, $localConf, "$key/$vhost/" );
+                }
+            }
         }
+
         else {
 
             # Check if key exists
@@ -1139,8 +1176,6 @@ sub _unitTest {
             if (   $key =~ /^(?:$simpleHashKeys|$doubleHashKeys)$/o
                 or $attr->{type} =~ /Container$/ )
             {
-                my $keyMsg = $attr->{keyMsgFail} // $type->{keyMsgFail};
-                my $msg    = $attr->{msgFail}    // $type->{msgFail};
                 $res = 0
                   unless (
                     $self->_execTest( {
@@ -1187,7 +1222,14 @@ sub _execTest {
     if ( $ref eq 'CODE' ) {
         my ( $r, $m ) = ( $test->( $value, $conf, $attr ) );
         if ($m) {
-            push @{ $self->{ ( $r ? 'warnings' : 'errors' ) } },
+            push @{
+                $self->{ (
+                        $r > 0
+                        ? 'warnings'
+                        : ( $r < 0 ? 'needConfirmation' : 'errors' )
+                    )
+                }
+              },
               { message => "$key: $m" };
         }
         elsif ( !$r ) {
@@ -1227,16 +1269,20 @@ sub _execTest {
 #
 #@return true if tests succeed
 sub _globalTest {
-    my $self = shift;
+    my ( $self, $localConf ) = @_;
+
     require Lemonldap::NG::Manager::Conf::Tests;
     hdebug('# _globalTest()');
     my $result = 1;
     my $tests  = &Lemonldap::NG::Manager::Conf::Tests::tests( $self->newConf );
+
     foreach my $name ( keys %$tests ) {
-        if (    $self->{skippedGlobalTests}
-            and $self->{skippedGlobalTests} =~ /\b$name\b/ )
+        if (    $localConf
+            and $localConf->{skippedGlobalTests}
+            and $localConf->{skippedGlobalTests} =~ /\b$name\b/ )
         {
-            $self->logger->debug("Ignore test for $name");
+            $localConf->logger->debug("-> Ignore test for $name\n")
+              if $localConf;
             next;
         }
         my $sub = $tests->{$name};
@@ -1258,7 +1304,7 @@ sub _globalTest {
         };
         if ($@) {
             push @{ $self->warnings }, "Test $name failed: $@";
-            print STDERR "Test $name failed: $@\n";
+            $localConf->logger->debug("Test $name failed: $@\n") if $localConf;
         }
     }
     return $result;
