@@ -4,7 +4,7 @@ use utf8;
 use Lemonldap::NG::Common::Regexp;
 use Lemonldap::NG::Handler::Main;
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.8';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -42,14 +42,8 @@ sub tests {
         # Check if portal URL is well formated
         portalURL => sub {
 
-            # Checking for ending slash
-            $conf->{portal} .= '/'
-              unless ( $conf->{portal} =~ qr#/$# );
-
-            # Deleting trailing ending slash
-            my $regex = qr#/+$#;
-            $conf->{portal} =~ s/$regex/\//;
-
+            # Append or remove trailing ending slashes
+            $conf->{portal} =~ s%/*$%/%;
             return 1;
         },
 
@@ -288,7 +282,7 @@ sub tests {
             return 1;
         },
 
-        # Error if session Activity Timeout is equal or lower than one minute
+    # Error if activity timeout interval is higher than session activity timeout
         timeoutActivityInterval => sub {
             return 1 unless ( defined $conf->{timeoutActivityInterval} );
             return ( 0,
@@ -404,6 +398,16 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        # Test if SAML private and public keys signature keys are set
+        samlSecretKeys => sub {
+            return 1 unless ( $conf->{issuerDBSAMLActivation} );
+            return ( 0,
+                'SAML service private and public keys signature must be set' )
+              unless ( $conf->{samlServicePrivateKeySig}
+                && $conf->{samlServicePublicKeySig} );
+            return 1;
+        },
+
         # Try to parse combination with declared modules
         checkCombinations => sub {
             return 1 unless ( $conf->{authentication} eq 'Combination' );
@@ -443,7 +447,6 @@ sub tests {
 
         # Warn if 2F dependencies seem missing
         sfaDependencies => sub {
-
             my $ok = 0;
             foreach (qw(u totp utotp yubikey)) {
                 $ok ||= $conf->{ $_ . '2fActivation' };
@@ -584,7 +587,7 @@ sub tests {
             return 1;
         },
 
-        # Warn if XSRF token TTL is higher than 10s
+        # Warn if XSRF token TTL is higher than 30s
         formTimeout => sub {
             return 1 unless ( defined $conf->{formTimeout} );
             return ( 0, "XSRF form token TTL must be higher than 30s" )
@@ -596,6 +599,18 @@ sub tests {
             return 1;
         },
 
+        # Warn if issuers token TTL is higher than 30s
+        issuersTimeout => sub {
+            return 1 unless ( defined $conf->{issuersTimeout} );
+            return ( 0, "Issuers token TTL must be higher than 30s" )
+              unless ( $conf->{issuersTimeout} > 30 );
+            return ( 1, "Issuers token TTL should not be higher than 2mn" )
+              if ( $conf->{issuersTimeout} > 120 );
+
+            # Return
+            return 1;
+        },
+
         # Warn if number of password reset retries is null
         passwordResetRetries => sub {
             return 1 unless ( $conf->{portalDisplayResetPassword} );
@@ -606,6 +621,19 @@ sub tests {
             return 1;
         },
 
+        # Warn if ldapPpolicyControl is used with AD (#2007)
+
+        ppolicyAd => sub {
+            if (    $conf->{ldapPpolicyControl}
+                and $conf->{authentication} eq "AD" )
+            {
+                return ( 1,
+"LDAP password policy control should be disabled when using AD authentication"
+                );
+            }
+            return 1;
+        },
+
         # Warn if bruteForceProtection enabled without History
         bruteForceProtection => sub {
             return 1 unless ( $conf->{bruteForceProtection} );
@@ -624,7 +652,7 @@ sub tests {
         checkMailResetSecurity => sub {
             return 1 unless ( $conf->{portalDisplayResetPassword} );
             return ( -1,
-'"passwordMailReset" plugin is enabled without CSRF Token neither Captcha required !!!'
+'"passwordMailReset" plugin is enabled without CSRF Token neither Captcha required'
               )
               unless ( $conf->{requireToken}
                 or $conf->{captcha_mail_enabled} );
@@ -633,16 +661,123 @@ sub tests {
             return 1;
         },
 
-        ## Warn if IdSpoofing plugin is enabled
-        # checkIdSpoofing => sub {
-        #     return ( -1,
-        #         '"IdSpoofing" plugin is enabled!!!'
-        #         )
-        #         if ( $conf->{idSpoofingRule} );
+        # Warn if Impersonation and ContextSwitching are simultaneously enabled
+        impersonation => sub {
+            return ( 1,
+                "Impersonation and ContextSwitching are simultaneously enabled"
+              )
+              if ( $conf->{impersonationRule}
+                && $conf->{contextSwitchingRule} );
 
-        #     # Return
-        #     return 1;
-        # },
+            # Return
+            return 1;
+        },
+
+# Warn if persistent storage is disabled with 2FA, History, OIDCConsents, Notifications or BruteForce protection
+        persistentStorage => sub {
+            return 1 unless ( $conf->{disablePersistentStorage} );
+            return ( 1, "2FA enabled WITHOUT persistent session storage" )
+              if ( $conf->{totp2fActivation}
+                || $conf->{yubikey2fActivation}
+                || $conf->{u2fActivation}
+                || $conf->{utotp2fActivation} );
+            return ( 1,
+                "History plugin enabled WITHOUT persistent session storage" )
+              if ( $conf->{loginHistoryEnabled} );
+            return ( 1,
+                "OIDC consents enabled WITHOUT persistent session storage" )
+              if ( $conf->{portalDisplayOidcConsents} );
+            return ( 1,
+                "Notifications enabled WITHOUT persistent session storage" )
+              if ( $conf->{notification} );
+            return ( 1,
+"BruteForceProtection plugin enabled WITHOUT persistent session storage"
+            ) if ( $conf->{bruteForceProtection} );
+
+            # Return
+            return 1;
+        },
+
+        # Warn if XML dependencies seem missing
+        xmlDependencies => sub {
+            return 1 unless ( $conf->{oldNotifFormat} );
+            eval "use XML::LibXML";
+            return ( 1,
+"XML::LibXML module is required to enable old format notifications"
+            ) if ($@);
+            eval "use XML::LibXSLT";
+            return ( 1,
+"XML::LibXSLT module is required to enable old format notifications"
+            ) if ($@);
+
+            # Return
+            return 1;
+        },
+
+        # Warn if CertificateResetByMail dependencies seem missing
+        certResetByMailDependencies => sub {
+            return 1 unless ( $conf->{portalDisplayCertificateResetByMail} );
+            return ( 0,
+"LDAP RegisterDB is required to enable CertificateResetByMail plugin"
+            ) unless ( $conf->{registerDB} eq 'LDAP' );
+            eval "use DateTime::Format::RFC3339";
+            return ( 1,
+"DateTime::Format::RFC3339 module is required to enable CertificateResetByMail plugin"
+            ) if ($@);
+
+            # Return
+            return 1;
+        },
+
+        # OIDC redirect URI must not be empty
+        oidcRPRedirectURINotEmpty => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} } );
+            my @msg;
+            my $res = 1;
+            foreach my $oidcRpId ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                unless ( $conf->{oidcRPMetaDataOptions}->{$oidcRpId}
+                    ->{oidcRPMetaDataOptionsRedirectUris} )
+                {
+                    push @msg,
+                      "$oidcRpId OpenID Connect RP has no redirect URI defined";
+                    $res = 0;
+                    next;
+                }
+            }
+            return ( $res, join( ', ', @msg ) );
+        },
+
+        # RS* OIDC algs require a signing key
+        oidcRPNeedRSAKey => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} } );
+            my @usingRSA = grep {
+                $conf->{oidcRPMetaDataOptions}->{$_}
+                  ->{oidcRPMetaDataOptionsIDTokenSignAlg}
+                  and $conf->{oidcRPMetaDataOptions}->{$_}
+                  ->{oidcRPMetaDataOptionsIDTokenSignAlg} =~ /^RS/
+            } keys %{ $conf->{oidcRPMetaDataOptions} };
+
+            if ( @usingRSA and not $conf->{oidcServicePrivateKeySig} ) {
+                my $msg =
+                  join( ", ", @usingRSA )
+                  . ": using RS-type encryption, but no RSA key is defined in global OIDC configuration";
+                return ( 0, $msg );
+            }
+            return 1;
+        },
+
+        # Notification system required with removed SF notification
+        sfRemovedNotification => sub {
+            return 1 unless ( $conf->{sfRemovedMsgRule} );
+            return ( 1,
+'Notification system must be enabled to display a notification if a SF is removed'
+            ) if ( $conf->{sfRemovedUseNotif} and not $conf->{notification} );
+            return 1;
+        },
     };
 }
 
