@@ -13,12 +13,15 @@ require Lemonldap::NG::Common::Notifications;
 
 use feature 'state';
 
-extends 'Lemonldap::NG::Common::Conf::AccessLib';
+extends qw(
+  Lemonldap::NG::Manager::Plugin
+  Lemonldap::NG::Common::Conf::AccessLib
+  Lemonldap::NG::Common::PSGI::Router
+);
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 has notifAccess => ( is => 'rw' );
-
 has notifFormat => ( is => 'rw' );
 
 #############################
@@ -27,7 +30,7 @@ has notifFormat => ( is => 'rw' );
 
 use constant defaultRoute => 'notifications.html';
 
-sub addRoutes {
+sub init {
     my ( $self, $conf ) = @_;
 
     if ( $conf->{oldNotifFormat} ) {
@@ -75,7 +78,7 @@ sub addRoutes {
           { done => { ':notificationId' => 'deleteDoneNotification' } },
         ['DELETE']
       );
-
+    return 1;
 }
 
 sub setNotifAccess {
@@ -158,6 +161,7 @@ sub notifications {
     my ( $notifs, $res );
 
     $notifs = $self->notifAccess->$sub();
+    my $total = ( keys %$notifs );
 
     # Restrict to wanted values
     if (
@@ -201,6 +205,7 @@ sub notifications {
                 result => 1,
                 count  => $count,
                 values => $res,
+                total  => $total
             }
         );
     }
@@ -220,8 +225,15 @@ sub notifications {
                 @r = sort { $a->{$f} cmp $b->{$f} } @r;
             }
         }
-        return $self->sendJSONresponse( $req,
-            { result => 1, count => scalar(@r), values => \@r } );
+        return $self->sendJSONresponse(
+            $req,
+            {
+                result => 1,
+                count  => scalar(@r),
+                values => \@r,
+                total  => $total
+            }
+        );
     }
 }
 
@@ -262,6 +274,8 @@ sub newNotification {
         return $self->sendError( $req, undef, 200 );
     }
 
+    $json->{reference} =~ s/_/-/g;    # Remove underscores (#2135)
+
     foreach my $r (qw(uid reference xml)) {
         return $self->sendError( $req, "Missing $r", 200 )
           unless ( $json->{$r} );
@@ -289,6 +303,7 @@ sub newNotification {
     $self->logger->debug("Notification Date = $json->{date}");
 
     unless ( $json->{date} =~ /^\d{4}-\d{2}-\d{2}$/ ) {
+        $self->logger->error("Malformed date");
         return $self->sendError( $req, "Malformed date", 200 );
     }
 
@@ -312,8 +327,8 @@ sub newNotification {
             delete $json->{xml};
         };
         if ($@) {
-            $self->logger->error("Notification malformed $@");
-            return $self->sendError( $req, "Notification malformed: $@", 200 );
+            $self->logger->error("Malformed notification $@");
+            return $self->sendError( $req, "Malformed notification: $@", 200 );
         }
         $newNotif = to_json($json);
     }
